%% Script to perform analysis of the VEC model.
%----Author: Guru Raghav Raman----
%Run after executing the script "Load_data.m".

%% Vector Error Correction (VEC) models
target_dates=84:158; %Before Circuit breaker: 23 Jan-6 April [PERIOD-1]
% target_dates=99:158; %DORSCON changed to orange: 7 Feb-6 April [PERIOD-2]
% target_dates=159:213; %Circuit breaker begins on 7 Apr: 7 Apr-31 May [PERIOD-3]

%Prerequisites
%Difference the time-series data to make them stationairy
input_1=diff(peak_total_demand(target_dates)');
input_2=diff(case_data(target_dates,1));
input_3=diff(case_data(target_dates,2));
input_4=mean(weather_1,1)';
input_5=mean(weather_2,1)';
input_4=diff(input_4(target_dates));
input_5=diff(input_5(target_dates));
input_set=[input_1 input_2 input_3 input_4 input_5];

%This is the set of original inputs, used for co-integration tests only.
weather_term_1=mean(weather_1,1)';
weather_term_2=mean(weather_2,1)';
input_original=[peak_total_demand(target_dates)' case_data(target_dates,1) case_data(target_dates,2) weather_term_1(target_dates) weather_term_2(target_dates)];

%Check for stationarity of the different time series data using Augmented
%Dickey-Fuller test.
[h1,p_value1]=adftest(input_1);
[h2,p_value2]=adftest(input_2);
[h3,p_value3]=adftest(input_3);
[h4,p_value4]=adftest(input_4);
[h5,p_value5]=adftest(input_5);

if (p_value1>0.05 || p_value2>0.05 || p_value3>0.05 || p_value4>0.05 || p_value5>0.05 || h1*h2*h3*h4*h5==0)
    disp('INPUT NOT STATIONARY');
end

%Johansen cointegration test
[h,pValue,stat,cValue,mles] = jcitest(input_original); 

%Number of cointegrating relations found from the Johansen cointegration test.
if h.r0==0
    rank_vecm=0;
elseif h.r1==0
    rank_vecm=1;
elseif h.r2==0
    rank_vecm=2;
elseif h.r3==0
    rank_vecm=3;
elseif h.r4==0
    rank_vecm=4;    
end

%Find out which parameters work the best and fit vector autoregression.
numseries = 5; %Number of series in the input
numlags = (1:7)'; %Which model size (lags) to test for the best one.
nummdls = numel(numlags);
T=size(input_1,1);

% Partition time base.
maxp = max(numlags);
idxpre = 1:maxp;
idxest = (maxp + 1):T;

% Preallocation
EstMdl(nummdls) = vecm(numseries,rank_vecm,0);
aic = zeros(nummdls,1);

% Fit models to data.
Y0 = input_set(idxpre,:);
Y = input_set(idxest,:);
for j = 1:numel(numlags)
    Mdl = vecm(numseries,rank_vecm,(numlags(j)-1));
    Mdl.SeriesNames = {{'peakdemand'},{'newcases'},{'recoveredcases'},{'weather1'},{'weather2'}};
    EstMdl(j) = estimate(Mdl,Y,'Y0',Y0);
    results = summarize(EstMdl(j));
    aic(j) = results.AIC;
end

[~,bestidx] = min(aic);

%Best model
BestMdl = EstMdl(bestidx);

%Forecast Error Variance Decomposition (EVD)
time_steps=100; %No. of timesteps to forecast error variance decomposition.
Decomposition = fevd(BestMdl,'NumObs',time_steps);

%% Plot FEVD results
color11='#e41a1c';
color12='#377eb8';
color13='#4daf4a';
color14='#984ea3';
color15='#ff7f00';

figure(5)
plot(1:time_steps,100*Decomposition(:,1,1),'-','MarkerSize',5,'LineWidth',3,'Color',color11);
hold on
plot(1:time_steps,100*Decomposition(:,2,1),'-','MarkerSize',5,'LineWidth',3,'Color',color12);
plot(1:time_steps,100*Decomposition(:,3,1),'-','MarkerSize',5,'LineWidth',3,'Color',color13);
plot(1:time_steps,100*Decomposition(:,4,1),'-','MarkerSize',5,'LineWidth',3,'Color',color14);
plot(1:time_steps,100*Decomposition(:,5,1),'-','MarkerSize',5,'LineWidth',3,'Color',color15);
xlabel('Forecast horizon (days)');
ylabel('Variance contribution (%)');
legend('Peak aggregate demand','New cases','Recovered cases','Weather component 1','Weather component 2');
box off